##############
# Dr. Albert Vernon Smith made a great contribution to improve
# this file.
#
#Albert Vernon Smith, Ph.D.
#Cold Spring Harbor Laboratory
#HapMap Data Coordination Center
#E-mail: smithav@cshl.edu
#
##############

## get parental data
getFounders <- function(ped)
{
  ## get parental info
  mat1 <- ped[, c(3:4)]
  tmp <- mat1[, 1]+mat1[, 2]
  pos <- which(tmp==0)
  if(length(pos)>0)
    {
      mat2 <- matrix(ped[pos, ], nrow=length(pos))
    }
  else 
    stop("The pedigree data contains no parental information!\n")
  return(mat2)
}

# In LdMax.cpp, the alleles need to be numbered sequentially up from one 
# Hence, we relabel the alleles.
#
# the most frequent allele is relabelled as allele 1
#
# genotypeMat -- n * (2*numMarkers) matrix
reLabelGenotypeMat<-function(genotypeMat)
{
  nrow<-nrow(genotypeMat)
  ncol<-ncol(genotypeMat)
  
  numMarkers<-ncol/2
  genotypeVec<-as.vector(genotypeMat) # stack column by column

  # stack two columns of a marker into one column
  genotypeMat2<-matrix(genotypeVec, ncol=numMarkers)

  tmp<-apply(genotypeMat2, 2, reLabelAlleles)
  
  tmp2<-as.vector(tmp)
  # retore to a 'nrow' by 'ncol' matrix
  res<-matrix(tmp2, ncol=ncol)
  return(res) 
}

# the most frequent allele is relabelled as allele 1
#
# genotype is a nx1 vector which stacks the two columns (loci) of alleles of 
# a marker 
# n is the number of subjects
reLabelAlleles<-function(genotype)
{
  genotypeVec<-as.vector(genotype)
  pos<-which(genotypeVec==0)
  genotypeVec2<-genotypeVec
  if(sum(pos)>0)
  { genotypeVec2<-genotypeVec2[-pos] }

  # get allele counts
  counts<-tapply(genotypeVec2, genotypeVec2, length)
  counts.s<-sort(counts, dec=TRUE)
  myla<-as.numeric(names(counts.s))
  
  len.u<-length(myla)

  genotypeVec3<-genotypeVec
  # the most frequent allele is relabelled as allele 1
  for(i in 1:len.u)
  {
    m<-myla[i]
    genotypeVec3[genotypeVec==m]<- i 
  }
  #genotype2<-matrix(genotypeVec3, ncol=2)
  genotype2<-genotypeVec3
  return(genotype2)
}


### get pair-wise LD measurment r^2  for founders of a ped matrix

getLD<-function(ped, columns=c("family", "pid", "father", "mother", "sex", "affected"), pooling.threshold=0.1, iprint=TRUE)
{
  len <- length(columns)

  ## get marker names
  pedColNames <- colnames(ped)
  pedColNames2 <- pedColNames[-(1:len)]
  tmplabels <- unlist(strsplit(pedColNames2, "\\."))
  tmplen <- length(tmplabels)
  markerNames <- tmplabels[seq(from=1, to=tmplen, by=4)]

  numMarkers<-length(markerNames)

  if(len>5 && prod(is.numeric(ped[,3])*is.numeric(ped[,4]))==TRUE)
  {
    # get founders genotype information
    pedFounders<-getFounders(ped)
  } else {
    pedFounders<-ped
  }
  founders<-pedFounders[,-(1:len), drop=FALSE]
  founders<-apply(founders, 2, as.numeric)
  nrow<-nrow(founders)
  ncol<-ncol(founders)

  res<-getLD.default(founders, nrow, ncol, numMarkers, pooling.threshold, 
                     iprint)
  return(res)

}

### get pair-wise LD measurment r^2  for founders of a ped matrix

getLDband<-function(ped, columns=c("family", "pid", "father", "mother", "sex", "affected"), width=31, pooling.threshold=0.1, iprint=TRUE)
{
  len <- length(columns)

  ## get marker names
  pedColNames <- colnames(ped)
  pedColNames2 <- pedColNames[-(1:len)]
  tmplabels <- unlist(strsplit(pedColNames2, "\\."))
  tmplen <- length(tmplabels)
  markerNames <- tmplabels[seq(from=1, to=tmplen, by=4)]

  numMarkers<-length(markerNames)

  if(len>5 && prod(is.numeric(ped[,3])*is.numeric(ped[,4]))==TRUE)
  {
    # get founders genotype information
    pedFounders<-getFounders(ped)
  } else {
    pedFounders<-ped
  }
  founders<-pedFounders[,-(1:len), drop=FALSE]
  founders<-apply(founders, 2, as.numeric)
  nrow<-nrow(founders)
  ncol<-ncol(founders)

  res<-getLDband.default(founders, nrow, ncol, numMarkers, width, pooling.threshold, iprint)
  return(res)

}

getLDdist<-function(ped, posVec, maxDist, columns=c("family", "pid", "father", "mother", "sex", "affected"), 
                    pooling.threshold=0.1, iprint=TRUE)
{
  len <- length(columns)

  ## get marker names
  pedColNames <- colnames(ped)
  pedColNames2 <- pedColNames[-(1:len)]
  tmplabels <- unlist(strsplit(pedColNames2, "\\."))
  tmplen <- length(tmplabels)
  markerNames <- tmplabels[seq(from=1, to=tmplen, by=4)]

  numMarkers<-length(markerNames)

  if(len>5 && prod(is.numeric(ped[,3])*is.numeric(ped[,4]))==TRUE)
  {
    # get founders genotype information
    pedFounders<-getFounders(ped)
  } else {
    pedFounders<-ped
  }
  founders<-pedFounders[,-(1:len), drop=FALSE]
  founders<-apply(founders, 2, as.numeric)
  nrow<-nrow(founders)
  ncol<-ncol(founders)

  res<-getLDdist.default(founders, nrow, ncol, numMarkers, posVec, maxDist, pooling.threshold, iprint)
  return(res)

}

getLD.default<-function(founders, nr, nc, numMarkers, pooling.threshold=0.1, iprint=TRUE)
{

  # the most frequent allele is relabelled as allele 1
  founders<-reLabelGenotypeMat(founders)

  founders<-as.integer(as.vector(t(founders)))
  nr<-as.integer(nr)
  nc<-as.integer(nc)
  pooling.threshold<-as.double(pooling.threshold)
  numMarkers<-as.integer(numMarkers)
  iprint<-as.integer(iprint)
  iprint2<-as.integer(FALSE)
  
  tmpn<-(numMarkers-1)*numMarkers/2

  res<-.C("getLD", founders, nr, nc, numMarkers, pooling.threshold, 
    iprint2, 
    rMat=as.double(rep(6, numMarkers*numMarkers)), 
    r2Mat=as.double(rep(6, numMarkers*numMarkers)), 
    DprimeMat=as.double(rep(6, numMarkers*numMarkers)), 
    DMat=as.double(rep(6, numMarkers*numMarkers)), 
    chisqMat=as.double(rep(-6, numMarkers*numMarkers)), 
    pvalMat=as.double(rep(6, numMarkers*numMarkers)), 
    nMat=as.integer(rep(-6, numMarkers*numMarkers)), 
    LODMat=as.double(rep(-6, numMarkers*numMarkers))) 

  pos<-which(res$rMat==6)
  if(length(pos)>0)
  { res$rMat[pos]<-NA }
  pos<-which(res$r2Mat==6)
  if(length(pos)>0)
  { res$r2Mat[pos]<-NA }
  pos<-which(res$DprimeMat==6)
  if(length(pos)>0)
  { res$DprimeMat[pos]<-NA }
  pos<-which(res$DMat==6)
  if(length(pos)>0)
  { res$DMat[pos]<-NA }
  pos<-which(res$chisqMat==-6)
  if(length(pos)>0)
  { res$chisqMat[pos]<-NA }
  pos<-which(res$pvalMat==6)
  if(length(pos)>0)
  { res$pvalMat[pos]<-NA }
  pos<-which(res$nMat==-6)
  if(length(pos)>0)
  { res$nMat[pos]<-NA }
  pos<-which(res$LODMat==-6)
  if(length(pos)>0)
  { res$LODMat[pos]<-NA }


  rMat<-matrix(res$rMat, nrow=numMarkers, ncol=numMarkers)
  r2Mat<-matrix(res$r2Mat, nrow=numMarkers, ncol=numMarkers)
  DprimeMat<-matrix(res$DprimeMat, nrow=numMarkers, ncol=numMarkers)
  DMat<-matrix(res$DMat, nrow=numMarkers, ncol=numMarkers)
  chisqMat<-matrix(res$chisqMat, nrow=numMarkers, ncol=numMarkers)
  pvalMat<-matrix(res$pvalMat, nrow=numMarkers, ncol=numMarkers)
  nMat<-matrix(res$nMat, nrow=numMarkers, ncol=numMarkers)
  LODMat<-matrix(res$LODMat, nrow=numMarkers, ncol=numMarkers)

  invisible(list(rMat=rMat, r2Mat=r2Mat, DprimeMat=DprimeMat, DMat=DMat, chisqMat=chisqMat, pvalMat=pvalMat, nMat=nMat,
                 LODMat=LODMat))
}


getLDband.default<-function(founders, nr, nc, numMarkers, width, 
                            pooling.threshold=0.1, iprint=TRUE)
{

  founders<-as.integer(as.vector(t(founders)))
  nr<-as.integer(nr)
  nc<-as.integer(nc)
  pooling.threshold<-as.double(pooling.threshold)
  numMarkers<-as.integer(numMarkers)
  iprint<-as.integer(iprint)
  iprint2<-as.integer(FALSE)
  width<-as.integer(width)

  lag <- floor(width/2)
  lag<-as.integer(lag)
  
  tmpn<-(numMarkers-1)*numMarkers/2

  res<-.C("getLDband", founders, nr, nc, numMarkers, width, lag, 
    pooling.threshold, iprint2, 
    rMat=as.double(rep(6, numMarkers*width)), 
    r2Mat=as.double(rep(6, numMarkers*width)), 
    DprimeMat=as.double(rep(6, numMarkers*width)), 
    DMat=as.double(rep(6, numMarkers*width)), 
    chisqMat=as.double(rep(-6, numMarkers*width)), 
    pvalMat=as.double(rep(6, numMarkers*width)), 
    nMat=as.integer(rep(-6, numMarkers*width)), 
    LODMat=as.double(rep(-6, numMarkers*width))) 

  pos<-which(res$rMat==6)
  if(length(pos)>0)
  { res$rMat[pos]<-NA }
  pos<-which(res$r2Mat==6)
  if(length(pos)>0)
  { res$r2Mat[pos]<-NA }
  pos<-which(res$DprimeMat==6)
  if(length(pos)>0)
  { res$DprimeMat[pos]<-NA }
  pos<-which(res$DMat==6)
  if(length(pos)>0)
  { res$DMat[pos]<-NA }
  pos<-which(res$chisqMat==-6)
  if(length(pos)>0)
  { res$chisqMat[pos]<-NA }
  pos<-which(res$pvalMat==6)
  if(length(pos)>0)
  { res$pvalMat[pos]<-NA }
  pos<-which(res$nMat==-6)
  if(length(pos)>0)
  { res$nMat[pos]<-NA }
  pos<-which(res$LODMat==-6)
  if(length(pos)>0)
  { res$LODMat[pos]<-NA }

  rMat<-matrix(res$rMat, nrow=numMarkers, ncol=width)
  r2Mat<-matrix(res$r2Mat, nrow=numMarkers, ncol=width)
  DprimeMat<-matrix(res$DprimeMat, nrow=numMarkers, ncol=width)
  DMat<-matrix(res$DMat, nrow=numMarkers, ncol=width)
  chisqMat<-matrix(res$chisqMat, nrow=numMarkers, ncol=width)
  pvalMat<-matrix(res$pvalMat, nrow=numMarkers, ncol=width)
  nMat<-matrix(res$nMat, nrow=numMarkers, ncol=width)
  LODMat<-matrix(res$LODMat, nrow=numMarkers, ncol=width)

  invisible(list(rMat=rMat, r2Mat=r2Mat, DprimeMat=DprimeMat, DMat=DMat, chisqMat=chisqMat, pvalMat=pvalMat, 
                 nMat=nMat, LODMat=LODMat))
}

getLDdist.default<-function(founders, nr, nc, numMarkers, posVec, maxDist, pooling.threshold=0.1, iprint=TRUE)
{

  # the most frequent allele is relabelled as allele 1
  founders<-reLabelGenotypeMat(founders)

  founders<-as.integer(as.vector(t(founders)))
  nr<-as.integer(nr)
  nc<-as.integer(nc)
  pooling.threshold<-as.double(pooling.threshold)
  numMarkers<-as.integer(numMarkers)
  iprint<-as.integer(iprint)
  iprint2<-as.integer(FALSE)
  posVec<-as.integer(posVec)
  maxDist<-as.integer(maxDist)
  
  tmpn<-(numMarkers-1)*numMarkers/2

  res<-.C("getLDdist", founders, nr, nc, numMarkers, posVec, maxDist,
    pooling.threshold, iprint2, 
    rMat=as.double(rep(6, numMarkers*numMarkers)), 
    r2Mat=as.double(rep(6, numMarkers*numMarkers)), 
    DprimeMat=as.double(rep(6, numMarkers*numMarkers)), 
    DMat=as.double(rep(6, numMarkers*numMarkers)), 
    chisqMat=as.double(rep(-6, numMarkers*numMarkers)), 
    pvalMat=as.double(rep(6, numMarkers*numMarkers)), 
    nMat=as.integer(rep(-6, numMarkers*numMarkers)), 
    LODMat=as.double(rep(-6, numMarkers*numMarkers))) 

  pos<-which(res$rMat==6)
  if(length(pos)>0)
  { res$rMat[pos]<-NA }
  pos<-which(res$r2Mat==6)
  if(length(pos)>0)
  { res$r2Mat[pos]<-NA }
  pos<-which(res$DprimeMat==6)
  if(length(pos)>0)
  { res$DprimeMat[pos]<-NA }
  pos<-which(res$DMat==6)
  if(length(pos)>0)
  { res$DMat[pos]<-NA }
  pos<-which(res$chisqMat==-6)
  if(length(pos)>0)
  { res$chisqMat[pos]<-NA }
  pos<-which(res$pvalMat==6)
  if(length(pos)>0)
  { res$pvalMat[pos]<-NA }
  pos<-which(res$nMat==-6)
  if(length(pos)>0)
  { res$nMat[pos]<-NA }
  pos<-which(res$LODMat==-6)
  if(length(pos)>0)
  { res$LODMat[pos]<-NA }


  rMat<-matrix(res$rMat, nrow=numMarkers, ncol=numMarkers)
  r2Mat<-matrix(res$r2Mat, nrow=numMarkers, ncol=numMarkers)
  DprimeMat<-matrix(res$DprimeMat, nrow=numMarkers, ncol=numMarkers)
  DMat<-matrix(res$DMat, nrow=numMarkers, ncol=numMarkers)
  chisqMat<-matrix(res$chisqMat, nrow=numMarkers, ncol=numMarkers)
  pvalMat<-matrix(res$pvalMat, nrow=numMarkers, ncol=numMarkers)
  nMat<-matrix(res$nMat, nrow=numMarkers, ncol=numMarkers)
  LODMat<-matrix(res$LODMat, nrow=numMarkers, ncol=numMarkers)

  invisible(list(rMat=rMat, r2Mat=r2Mat, DprimeMat=DprimeMat, DMat=DMat, chisqMat=chisqMat, pvalMat=pvalMat, nMat=nMat,
                 LODMat=LODMat))
}

